// decoding APRS and MICE into WPL sentences  

 #include <LibAPRS.h>
 #include <string.h>
 #include <stdlib.h>
 
//#define ADC_REFERENCE REF_3V3
  #define ADC_REFERENCE REF_5V

  #define OPEN_SQUELCH false  //open
//#define OPEN_SQUELCH true   //closed

  #define Deg2Rad 0.0174532925 // value for calc. radians
 
String call, dest, contents, mice;
String lat, lon, ns, ew, txt, wpl;
int call_ssid;

byte index = 0;   
byte start_with = 0;
byte end_with = 48;   
byte CRC = 0;

/////////////////////////////////////////////////////////////
boolean gotPacket = false;
AX25Msg incomingPacket;
uint8_t *packetData;

////////////////////////////////////////////////////////////
void aprs_msg_callback(struct AX25Msg *msg) 
{
  if (!gotPacket) {
    gotPacket = true;
    memcpy(&incomingPacket, msg, sizeof(AX25Msg));
  if (freeMemory() > msg->len) {
      packetData = (uint8_t*)malloc(msg->len);
      memcpy(packetData, msg->info, msg->len);
      incomingPacket.info = packetData;
    } else {
      gotPacket = false;
    }
  }
}

/////////////////////////////////////////////////////////////
void setup() 
{
  Serial.begin(9600);
  APRS_init(ADC_REFERENCE, OPEN_SQUELCH);
}

///////////////////////////////////////////////////////////////
void processPacket() {
  if (gotPacket) {
    gotPacket = false;

    call = (incomingPacket.src.call);      
    call_ssid = (incomingPacket.src.ssid); 
    dest = (incomingPacket.dst.call); 

   for (int i = 0; i < incomingPacket.len; i++)   {
   contents=contents+char((incomingPacket.info[i]));  }      
   
//    APRS mode select
    if(contents.charAt(0)=='@'||contents.charAt(0)=='/')
    { contents = contents.substring(8); 
     use_aprs();  }

else if(contents.charAt(0)=='!'||contents.charAt(0)=='=')
    { contents = contents.substring(1); 
      use_aprs(); }

else if(contents.charAt(0)==char(39)||contents.charAt(0)==char(96))
    { contents = contents.substring(1); 
     use_mice(); }

  contents = "";
  free(packetData);
  }
}

/////////////////////////////////////////////////////////////
void loop() 
{
  processPacket();
}

//////////////////////////////////////////////////////////////
void use_aprs()
{
//    Serial.print("APRS:   ");
//    Serial.print(call);                 

 if (incomingPacket.src.ssid>0)
   { call.concat("-");
     call.concat(call_ssid); }  

  lat = contents.substring(0,7);
  ns = contents.substring(7,8);

  lon = contents.substring(9,17);
  ew = contents.substring(17,18);

// APRS Mode: building wpl sentences 
  wpl = "GPWPL,"; 
  wpl.concat(lat);
  wpl.concat(",");
  wpl.concat(ns);
  wpl.concat(",");
  wpl.concat(lon);
  wpl.concat(",");
  wpl.concat(ew);
  wpl.concat(",");
  wpl.concat(call);

// XOR for checksum calculation
    byte end_with = wpl.length();
    for (byte x = start_with; x<end_with; x++) {  // XOR every character 
    CRC = CRC ^ wpl[x] ;
    }

// add '$' at start and '*' at end as well as CRC top complete the string
    wpl = '$' + wpl + '*';
    Serial.print(wpl);
    if(CRC<16) 
  { Serial.print("0");  }
    Serial.println(CRC,HEX);
    CRC = 0;                              // reset CRC variable
    wpl= "";
}   

//////////////////////////////////////////////////////////////
void use_mice()
{
//    Serial.print("MICE:   ");
//    Serial.print(call);                 

 if (incomingPacket.src.ssid>0)
   { call.concat("-");
     call.concat(call_ssid); }  

// MICE: LAT decoding  
int lat1 = dest.charAt(0)&15;
int lat2 = dest.charAt(1)&15;
int lat3 = dest.charAt(2)&15;
int lat4 = dest.charAt(3)&15;
int lat5 = dest.charAt(4)&15;
int lat6 = dest.charAt(5)&15;

int grad_lat = (lat1*10)+lat2;
int dec_lat = (lat3*1000)+(lat4*100)+(lat5*10)+lat6;

// Serial.println(dec_lat);   //+++++++++

// MICE: flag decoding
int fg1 = dest.charAt(3)&64;
if(dest.charAt(3)==76) { fg1=0 ;}  
int fg2 = dest.charAt(4)&64;
if(dest.charAt(4)==76) { fg2=0 ;}  
int fg3 = dest.charAt(5)&64;
if(dest.charAt(5)==76) { fg3=0 ;}  

// MICE: N/S and E/W decoding
if(fg1==64) { ns="N"; }
else {ns="S"; }
if(fg3==64) { ew="W"; }
else {ew="E"; }

String str_grad_lat = String(grad_lat);
String str_dec_lat = String(dec_lat);

   if(grad_lat <10) 
   {str_grad_lat = "0" + str_grad_lat; }

   if(dec_lat <10) 
   {str_dec_lat = "0" + str_dec_lat; }
   if(dec_lat <100) 
   {str_dec_lat = "0" + str_dec_lat; }
   if(dec_lat <1000) 
   {str_dec_lat = "0" + str_dec_lat; }

String str_dec_lat1 = str_dec_lat.substring(0,2); 
String str_dec_lat2 = str_dec_lat.substring(2,4);

// MICE: LON decoding  
mice = contents.substring((0),(5));

int lon1a = mice.charAt(0);
int lon2a = mice.charAt(1);
int lon3a = mice.charAt(2);

int lon1b = lon1a-28;
int lon1c = lon1b;                        
if(fg2==64) { lon1b=(lon1b+100); lon1c=lon1b; }
if(lon1b>179&&lon1b<190) {lon1c=lon1b-80 ; }
if(lon1b>189&&lon1b<200) {lon1c=lon1b-190 ; }  
int grad_lon = lon1c;

int lon2b=lon2a-28;
if(lon2b>59) {lon2b=lon2b-60 ; }
int lon3b=lon3a-28;
int dec_lon = (lon2b*100)+lon3b;             

String str_grad_lon = String(grad_lon);
String str_dec_lon = String(dec_lon);

// Serial.println(dec_lon);   //+++++++++

   if(grad_lon <10) 
   {str_grad_lon = "0" + str_grad_lon; }
   if(grad_lon <100) 
   {str_grad_lon = "0" + str_grad_lon; }

   if(dec_lon <10) 
   {str_dec_lon = "0" + str_dec_lon; }
   if(dec_lon <100) 
   {str_dec_lon = "0" + str_dec_lon; }
   if(dec_lon <1000) 
   {str_dec_lon = "0" + str_dec_lon; }

String str_dec_lon1 = str_dec_lon.substring(0,2); 
String str_dec_lon2 = str_dec_lon.substring(2,4);

 // MICE: building wpl sentences 
  wpl = "GPWPL," ;
  wpl.concat(str_grad_lat);
  wpl.concat(str_dec_lat1);
  wpl.concat(".");
  wpl.concat(str_dec_lat2);
  wpl.concat(",");
  wpl.concat(ns);
  wpl.concat(",");
  wpl.concat(str_grad_lon);
  wpl.concat(str_dec_lon1);
  wpl.concat(".");
  wpl.concat(str_dec_lon2);
  wpl.concat(",");
  wpl.concat(ew);
  wpl.concat(",");
  wpl.concat(call);

// XOR for checksum calculation
    byte end_with = wpl.length();
    for (byte x = start_with; x<end_with; x++) {  // XOR every character 
    CRC = CRC ^ wpl[x] ;
    }

// add '$' at start and '*' at end as well as CRC top complete the string
    wpl = '$' + wpl + '*';
    Serial.print(wpl);
    if(CRC<16) 
  { Serial.print("0");  }
    Serial.println(CRC,HEX);
    CRC = 0;                                     // reset CRC variable
    wpl= "";
    str_grad_lon = "";
    str_dec_lat = "";
    str_grad_lon = "";
    str_dec_lon = "";
}


